package ps6.graph;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import junit.framework.TestCase;

/**
 * <code>AbstractMutableDirectedLabeledMultiGraphTest</code> is a test of an
 * implementation of {@link ps6.graph.MutableDirectedLabeledMultiGraph}.<p/>
 * Currently, this class tests all methods of
 * <code>AbstractMutableDirectedLabeledMultiGraphTest</code> on an empty graph
 * and on a complete graph with 5 nodes. This test suite could be improved by
 * adding test cases for asymmetric graphs, as well. <p/>To use this class to
 * test your own implementation of <code>MutableDirectedLabeledMultiGraph</code>,
 * extend this class and override {@link #createMultiGraph()}so that it returns
 * a new instance of your implementation. Then you can run your new class as a
 * JUnit test. <p/>This is useful because it allows you to have a separate unit
 * test for every implementation of the interface (in the event that there are
 * multiple implementations).
 * 
 * @author mbolin
 * @author cwo (modified for Java 1.5) 
 */
public abstract class AbstractMutableDirectedLabeledMultiGraphTest extends
		TestCase {

	/**
	 * Override this method so it returns the implementation of
	 * {@link ps6.graph.MutableDirectedLabeledMultiGraph}that you wish to test.
	 */
	protected abstract MutableDirectedLabeledMultiGraph<String> createMultiGraph();

	/**
	 * This JUnit test uses this method to create <code>Edge</code> objects.
	 * By default, it uses {@link ps6.graph.DefaultEdge}for all of its tests,
	 * but this method can be overridden so that a different implementation of
	 * <code>Edge</code> is used throughout.
	 */
	protected Edge<String> createEdge(String source, String sink, Object label) {
		return new DefaultEdge<String>(source, sink, label);
	}

	/**
	 * Graph with 0 nodes and 0 edges.
	 * 
	 * This gets instantiated in {@link #setUp()}
	 */
	protected MutableDirectedLabeledMultiGraph<String> emptyGraph;

	/**
	 * Graph with 5 nodes and 2 edges between each node; one in each direction.
	 * 
	 * This gets instantiated in {@link #setUp()}
	 */
	protected MutableDirectedLabeledMultiGraph<String> completeGraphWithFiveNodes;

	/*
	 * A,B,C,D,E are used as nodes in the graph. For convenience, we use
	 * strings, but any object should work so long as no pair from A,B,C,D,E are
	 * equal to one another.
	 */

	protected static final String A = "A";

	protected static final String B = "B";

	protected static final String C = "C";

	protected static final String D = "D";

	protected static final String E = "E";

	/*
	 * The five nodes above should be added to the completeNodes set
	 */

	protected Set<String> completeNodes;

	/*
	 * The next 20 objects are used as edge labels in the graph. Again, for
	 * convenience, we use strings, but any object should work so long as no two
	 * labels are equal to one another.
	 */

	protected static final String AB = "AB";

	protected static final String BA = "BA";

	protected static final String AC = "AC";

	protected static final String CA = "CA";

	protected static final String AD = "AD";

	protected static final String DA = "DA";

	protected static final String AE = "AE";

	protected static final String EA = "EA";

	protected static final String BC = "BC";

	protected static final String CB = "CB";

	protected static final String BD = "BD";

	protected static final String DB = "DB";

	protected static final String BE = "BE";

	protected static final String EB = "EB";

	protected static final String CD = "CD";

	protected static final String DC = "DC";

	protected static final String CE = "CE";

	protected static final String EC = "EC";

	protected static final String DE = "DE";

	protected static final String ED = "ED";

	/*
	 * The 20 edges above should be added to the completeEdges set
	 */

	protected Set<Edge> completeEdges;

	/**
	 * Initialize the emptyGraph and the completeGraphWithFiveNodes that are
	 * used throughout for testing.
	 */
	protected void setUp() throws Exception {
		emptyGraph = createMultiGraph();

		completeGraphWithFiveNodes = createMultiGraph();
		completeGraphWithFiveNodes.addNode(A);
		completeGraphWithFiveNodes.addNode(B);
		completeGraphWithFiveNodes.addNode(C);
		completeGraphWithFiveNodes.addNode(D);
		completeGraphWithFiveNodes.addNode(E);

		completeNodes = new HashSet<String>();
		completeNodes.add(A);
		completeNodes.add(B);
		completeNodes.add(C);
		completeNodes.add(D);
		completeNodes.add(E);

		completeGraphWithFiveNodes.addEdge(createEdge(A, B, AB));
		completeGraphWithFiveNodes.addEdge(createEdge(B, A, BA));
		completeGraphWithFiveNodes.addEdge(createEdge(A, C, AC));
		completeGraphWithFiveNodes.addEdge(createEdge(C, A, CA));
		completeGraphWithFiveNodes.addEdge(createEdge(A, D, AD));
		completeGraphWithFiveNodes.addEdge(createEdge(D, A, DA));
		completeGraphWithFiveNodes.addEdge(createEdge(A, E, AE));
		completeGraphWithFiveNodes.addEdge(createEdge(E, A, EA));
		completeGraphWithFiveNodes.addEdge(createEdge(B, C, BC));
		completeGraphWithFiveNodes.addEdge(createEdge(C, B, CB));
		completeGraphWithFiveNodes.addEdge(createEdge(B, D, BD));
		completeGraphWithFiveNodes.addEdge(createEdge(D, B, DB));
		completeGraphWithFiveNodes.addEdge(createEdge(B, E, BE));
		completeGraphWithFiveNodes.addEdge(createEdge(E, B, EB));
		completeGraphWithFiveNodes.addEdge(createEdge(C, D, CD));
		completeGraphWithFiveNodes.addEdge(createEdge(D, C, DC));
		completeGraphWithFiveNodes.addEdge(createEdge(C, E, CE));
		completeGraphWithFiveNodes.addEdge(createEdge(E, C, EC));
		completeGraphWithFiveNodes.addEdge(createEdge(D, E, DE));
		completeGraphWithFiveNodes.addEdge(createEdge(E, D, ED));

		completeEdges = new HashSet<Edge>();
		completeEdges.add(createEdge(A, B, AB));
		completeEdges.add(createEdge(B, A, BA));
		completeEdges.add(createEdge(A, C, AC));
		completeEdges.add(createEdge(C, A, CA));
		completeEdges.add(createEdge(A, D, AD));
		completeEdges.add(createEdge(D, A, DA));
		completeEdges.add(createEdge(A, E, AE));
		completeEdges.add(createEdge(E, A, EA));
		completeEdges.add(createEdge(B, C, BC));
		completeEdges.add(createEdge(C, B, CB));
		completeEdges.add(createEdge(B, D, BD));
		completeEdges.add(createEdge(D, B, DB));
		completeEdges.add(createEdge(B, E, BE));
		completeEdges.add(createEdge(E, B, EB));
		completeEdges.add(createEdge(C, D, CD));
		completeEdges.add(createEdge(D, C, DC));
		completeEdges.add(createEdge(C, E, CE));
		completeEdges.add(createEdge(E, C, EC));
		completeEdges.add(createEdge(D, E, DE));
		completeEdges.add(createEdge(E, D, ED));
	}

	public void testContainsNode() {
		// make sure empty graph does not contain any nodes
      String[] randomNodes = new String[] { "Hello", "World", "!" };
		for (int i = 0; i < randomNodes.length; i++) {
			assertFalse("empty graph has a node", emptyGraph.containsNode(randomNodes[i]));
		}

		// make sure complete graph contains its 5 nodes
		String[] nodes = new String[] { A, B, C, D, E };
		for (int i = 0; i < nodes.length; i++) {
			assertTrue("complete graph missing node",
					completeGraphWithFiveNodes.containsNode(nodes[i]));
		}

		// make sure complete graph does not contain other nodes
		for (int i = 0; i < randomNodes.length; i++) {
			assertFalse("complete graph should not contain a random node",
					completeGraphWithFiveNodes.containsNode(randomNodes[i]));
		}
	}

	public void testContainsEdge() {
      Edge<String> first = createEdge(A,B,AB);
      Edge<String> second = createEdge(B,A,BA);
      Edge<String> third = createEdge(C,E,CE);
      Edge<String> fourth = createEdge(E,D,ED);
      Edge<String> fifth = createEdge(B,D,BD);

		// make sure empty graph does not contain any edges
		assertFalse("empty graph has an edge", emptyGraph.containsEdge(first));
		assertFalse("empty graph has an edge", emptyGraph.containsEdge(second));
		assertFalse("empty graph has an edge", emptyGraph.containsEdge(third));
		assertFalse("empty graph has an edge", emptyGraph.containsEdge(fourth));
		assertFalse("empty graph has an edge", emptyGraph.containsEdge(fifth));

		// make sure complete graph has all edges
		assertTrue("complete graph is missing an edge ", completeGraphWithFiveNodes.containsEdge(first));
		assertTrue("complete graph is missing an edge ", completeGraphWithFiveNodes.containsEdge(second));
		assertTrue("complete graph is missing an edge ", completeGraphWithFiveNodes.containsEdge(third));
		assertTrue("complete graph is missing an edge ", completeGraphWithFiveNodes.containsEdge(fourth));
		assertTrue("complete graph is missing an edge ", completeGraphWithFiveNodes.containsEdge(fifth));

	}

	public void testGetNodes() {
		// make sure empty graph returns empty Iterator
		assertFalse("empty graph should have no nodes", emptyGraph.getNodes()
				.hasNext());

		// make sure elements in nodes Iterator are exactly the nodes in the
		// graph
		for (Iterator<String> nodes = completeGraphWithFiveNodes.getNodes(); nodes
				.hasNext();) {
			String node = nodes.next();
			assertTrue("node in nodes Iterator was not in graph", completeNodes
					.remove(node));
			assertTrue("graph has too many nodes", nodes.hasNext()
					^ completeNodes.isEmpty());
		}

	}

	public void testGetEdges() {
		// make sure empty graph returns empty Iterator
		assertFalse("empty graph should have 0 edges", emptyGraph.getEdges()
				.hasNext());

		// make sure elements in edges Iterator are exactly the edges in the
		// graph
		for (Iterator<Edge<String> > edges = completeGraphWithFiveNodes.getEdges(); edges
				.hasNext();) {
			Edge<String> edge = edges.next();
			assertTrue("edge in edges Iterator was not in graph", completeEdges
					.remove(edge));
			assertTrue("graph has too many edges", edges.hasNext()
					^ completeEdges.isEmpty());
		}
	}

	public void testGetNodeCount() {
		assertEquals("empty graph should have 0 nodes", 0, emptyGraph
				.getNodeCount());
		assertEquals("complete graph should have 5 nodes", 5,
				completeGraphWithFiveNodes.getNodeCount());
	}

	public void testGetEdgeCount() {
		assertEquals("empty graph should have 0 edges", 0, emptyGraph
				.getEdgeCount());
		assertEquals("complete graph should have 20 edges", 20,
				completeGraphWithFiveNodes.getEdgeCount());
	}

	public void testGetOutgoingEdges() {
		// empty graph should return empty Iterator for getOutgoingEdges
		assertFalse("empty graph has node with outgoing edges", emptyGraph
				.getOutgoingEdges(D).hasNext());

		// create a list of outgoing edges that the node has
		Set<Edge<String> > edges = new HashSet<Edge<String> >();
		edges.add(createEdge(B, A, BA));
		edges.add(createEdge(B, C, BC));
		edges.add(createEdge(B, D, BD));
		edges.add(createEdge(B, E, BE));

		// iterate over the outgoing edges, removing each edge from the set
		// above as it is found.
		// this makes it straightforward to test if any extra outgoing edges are
		// returned
		for (Iterator<Edge<String> > iter = completeGraphWithFiveNodes.getOutgoingEdges(B); iter
				.hasNext();) {
			Edge<String> e = iter.next();
			assertTrue("outgoing edge is missing from graph: " + e, edges
					.contains(e));
			edges.remove(e);
			assertTrue("graph has too many outgoing edges", iter.hasNext()
					^ edges.isEmpty());
		}
	}

	public void testGetIncomingEdges() {
		// empty graph should return empty Iterator for getIncomingEdges
		assertFalse("empty graph has node with outgoing edges", emptyGraph
				.getIncomingEdges(E).hasNext());

		// create a list of incoming edges that the node has
		Set<Edge<String>> edges = new HashSet<Edge<String>>();
		edges.add(createEdge(A, E, AE));
		edges.add(createEdge(B, E, BE));
		edges.add(createEdge(C, E, CE));
		edges.add(createEdge(D, E, DE));

		// iterate over the incoming edges, removing each edge from the set
		// above as it is found.
		// this makes it straightforward to test if any extra incoming edges are
		// returned
		for (Iterator<Edge<String> > iter = completeGraphWithFiveNodes.getIncomingEdges(E); iter
				.hasNext();) {
			Edge<String> e = iter.next();
			assertTrue("incoming edge is missing from graph: " + e, edges
					.contains(e));
			edges.remove(e);
			assertTrue("graph has too many incoming edges", iter.hasNext()
					^ edges.isEmpty());
		}
	}

	public void testGetOutgoingEdgeCount() {
		// make sure empty graph returns 0 for getOutgoingEdgeCount
		// even though node isn't in graph
		assertEquals("empty graph has nonzero number of outgoing edges", 0,
				emptyGraph.getOutgoingEdgeCount(A));

		// check outgoing count correct for nodes in complete graph
		for (Iterator<String> nodes = completeNodes.iterator(); nodes.hasNext();) {
			assertEquals("outgoing edge count incorrect", 4,
					completeGraphWithFiveNodes.getOutgoingEdgeCount(nodes
							.next()));
		}
	}

	public void testGetIncomingEdgeCount() {
		// make sure empty graph returns 0 for getIncomingEdgeCount
		// even though node isn't in graph
		assertEquals("empty graph has nonzero number of incoming edges", 0,
				emptyGraph.getOutgoingEdgeCount(B));

		for (Iterator<String> nodes = completeNodes.iterator(); nodes.hasNext();) {
			assertEquals("incoming edge count incorrect", 4,
					completeGraphWithFiveNodes.getIncomingEdgeCount(nodes.next()));
		}
	}

	public void testRemoveEdge() {
		completeGraphWithFiveNodes.removeEdge(createEdge(A, B, AB));
		assertFalse("should no longer contain edge from A to B",
				completeGraphWithFiveNodes.containsEdge(createEdge(A, B, AB)));
		assertEquals("A should have 3 outgoing edges", 3,
				completeGraphWithFiveNodes.getOutgoingEdgeCount(A));
		assertEquals("B should have 3 incoming edges", 3,
				completeGraphWithFiveNodes.getIncomingEdgeCount(B));

	}

	public void testRemoveNode() {
		completeGraphWithFiveNodes.removeNode(A);
		assertEquals("should only have 4 nodes now", 4,
				completeGraphWithFiveNodes.getNodeCount());
		assertEquals("should only have 16 edges now", 12,
				completeGraphWithFiveNodes.getEdgeCount());
	}

}
