package ps6.graph;

/**
 * <code>DefaultEdge</code> is a default implementation of the
 * {@link ps6.graph.Edge} specification.
 * 
 * @author mbolin
 */
public final class DefaultEdge<T> implements Edge<T> {

	private final T source;

	private final T sink;

	private final Object label;

	private final int hashCode;

	/**
	 * Creates a new edge with the specified source, sink, and label.
	 */
	public DefaultEdge(T source, T sink, Object label) {
		this.source = source;
		this.sink = sink;
		this.label = label;

		// cache the hash code as it will probably be requested often by a graph
		int hcode = 17;
		hcode = 31 * hcode + GraphUtils.hcode(getSource());
		hcode = 31 * hcode + GraphUtils.hcode(getLabel());
		hcode = 31 * hcode + GraphUtils.hcode(getSink());
		hashCode = hcode;
	}

	/**
	 * {@inheritDoc}
	 */	
	public T getSource() {
		return source;
	}

	/**
	 * {@inheritDoc}
	 */	
	public Object getLabel() {
		return label;
	}

	/**
	 * {@inheritDoc}
	 */	
	public T getSink() {
		return sink;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean equals(Object o) {
		if (!(o instanceof Edge)) {
			return false;
		}
		Edge e = (Edge) o;
		return GraphUtils.eq(this.getSource(), e.getSource())
				&& GraphUtils.eq(this.getLabel(), e.getLabel())
				&& GraphUtils.eq(this.getSink(), e.getSink());
	}

	/**
	 * {@inheritDoc}
	 */
	public int hashCode() {
		return hashCode;
	}

	/*
	 * Inherit default specification for toString() so client does not depend
	 * upon the String representation of the edge.
	 */

	/**
	 * {@inheritDoc}
	 */	
	public String toString() {
		// Though DefaultEdge is "immutable," it may be mutated in ways
		// that do not affect its equals() method, but do affect its
		// toString() method. Thus, the string representation of this
		// edge is not cached.
		StringBuffer sb = new StringBuffer();
		sb.append("DefaultEdge:source=");
		sb.append(String.valueOf(source));
		sb.append(",sink=");
		sb.append(String.valueOf(sink));
		sb.append(",label=");
		sb.append(String.valueOf(label));
		return sb.toString();
	}

}
