package ps6.model;

import junit.framework.TestCase;

/**
 * Blackbox test of BoggleBoard's specification.
 * 
 * @author mbolin
 */
public class BoggleBoardTest extends TestCase {

    protected BoggleBoard board = null;

    /**
     * Creates this board:
     * 
     * <pre>
     * N S P U
     * E O T R
     * W F I E
     * S P A P
     * </pre>
     * 
     */
    protected void setUp() throws Exception {
        CubeBucket[] buckets = new CubeBucket[] {
                new CubeBucket(0, Cube.CUBE_1, 2),
                new CubeBucket(1, Cube.CUBE_1, 4),
                new CubeBucket(2, Cube.CUBE_7, 2),
                new CubeBucket(3, Cube.CUBE_13, 5),
                new CubeBucket(4, Cube.CUBE_0, 0),
                new CubeBucket(5, Cube.CUBE_8, 4),
                new CubeBucket(6, Cube.CUBE_11, 5),
                new CubeBucket(7, Cube.CUBE_10, 4),
                new CubeBucket(8, Cube.CUBE_2, 5),
                new CubeBucket(9, Cube.CUBE_14, 1),
                new CubeBucket(10, Cube.CUBE_14, 2),
                new CubeBucket(11, Cube.CUBE_6, 0),
                new CubeBucket(12, Cube.CUBE_7, 3),
                new CubeBucket(13, Cube.CUBE_7, 2),
                new CubeBucket(14, Cube.CUBE_11, 0),
                new CubeBucket(15, Cube.CUBE_7, 2) };
        board = new BoggleBoard(buckets);
    }

    /**
     * This tests that this board:
     * 
     * <pre>
     * N S P U
     * E O T R
     * W F I E
     * S P A P
     * </pre>
     * 
     * contains the following words:
     * 
     * NEW
     * NEWS
     * PAPER
     * NEWSPAPER
     * PUT
     * RUT
     * RIP
     * FIT
     * TOP
     * POTS
     * STOP
     * POST
     * OPTS
     * TOPS
     * SPOT
     * SOFT
     * NEST
     * PIER
     * TRIP
     * 
     * but NOT these words:
     * 
     * NEWT
     * PURTY
     * TOT
     * TOSS
     * PAPST
     * FPAIF
     */
    public void testContainsWord1() {

        String[] includedWords = new String[] { "NEW", "NEWS", "PAPER",
                "NEWSPAPER", "PUT", "RUT", "RIP", "FIT", "TOP", "POTS", "STOP",
                "POST", "OPTS", "TOPS", "SPOT", "SOFT", "NEST", "PIER", "TRIP" };
        for (int i = 0; i < includedWords.length; i++) {
            assertTrue("does not contain word: " + includedWords[i], board
                    .containsWord(includedWords[i]));
        }

        String[] excludedWords = new String[] { "NEWT", "PURTY", "TOT", "TOSS",
                "PAPST", "FPAIF" };
        for (int i = 0; i < excludedWords.length; i++) {
            assertFalse("includes word: " + excludedWords[i], board
                    .containsWord(excludedWords[i]));
        }
    }

    /**
     * Tests to make sure that a Boggle board thinks it is of size 4.
     */
    public void testGetBoardSize() {
        assertEquals("boggle board should be of size 4", 4, board
                .getBoardSize());
    }

    /**
     * Tests {@link ps6.model.Board#getLettersAt(int,int)}
     * for all of the letters in this board:
     * 
     * <pre>
     * N S P U
     * E O T R
     * W F I E
     * S P A P
     * </pre>
     */
    public void testGetLettersAt() {
        String[][] letters = { { "N", "S", "P", "U" }, { "E", "O", "T", "R" },
                { "W", "F", "I", "E" }, { "S", "P", "A", "P" } };
        for (int row = 0; row < letters.length; row++) {
            for (int col = 0; col < letters[row].length; col++) {
                assertEquals("did not retrieve correct letter",
                        letters[row][col], board.getLettersAt(row, col));
            }
        }
    }

    /**
     * This makes sure that shuffle events get sent to shuffle listeners
     * and that events no longer get sent once the listener is removed.
     */
    public void testShuffleListeners() {
        TestShuffleListener listener = new TestShuffleListener();
        assertFalse("should not have heard event yet", listener.hasHeardEvent());

        board.addShuffleListener(listener);
        assertFalse("should not have heard event yet", listener.hasHeardEvent());

        board.shuffle();
        assertTrue("should have heard event", listener.hasHeardEvent());

        listener.reset();
        assertFalse("should not have heard event anymore after reset", listener
                .hasHeardEvent());

        board.removeShuffleListener(listener);
        board.shuffle();
        assertFalse("should not have heard event now that listener is removed",
                listener.hasHeardEvent());
    }

    /**
     * This is a trivial ShuffleListener used to test
     * if shuffle events get fired correctly.
     */
    private static class TestShuffleListener implements ShuffleListener {

        private boolean heardEvent = false;

        public void boardShuffled(Board b) {
            heardEvent = true;
        }

        public boolean hasHeardEvent() {
            return heardEvent;
        }

        public void reset() {
            heardEvent = false;
        }
    }

}