
package ps6.model;

import java.util.HashMap;
import java.util.Map;

/**
 * <code>Face</code> represents a face of a cube in Boggle.
 *
 * @specfield String displayLetters // the letters of the Face
 *                                    // as they should be displayed in a UI
 * 
 * @derivedfield String capitalLetters // the letters of the Face in all caps,
 *                                    // convenient for comparison
 *
 * @author mbolin
 */
public final class Face {

    /** maps capitalized version of letters to Face object -- provides Face interning */
    private static Map<String,Face> FACES = new HashMap<String,Face>();
    
    /** the letters that appear on the Face */
    private final String displayLetters;
    
    /** the letters that appear on the Face, in all caps */
    private final String capitalLetters;
    
    /**
     * @requires displayLetters != null
     */
    private Face(String displayLetters) {
        this.displayLetters = displayLetters;
        this.capitalLetters = displayLetters.toUpperCase();
        FACES.put(capitalLetters, this);
    }
    
    /**
     * @return a Face with the specified letters
     */
    public static Face createFace(String letters) {
        if (letters == null) {
            throw new IllegalArgumentException("letters on face cannot be null");
        }
        Face face = (Face)FACES.get(letters.toUpperCase());
        if (face != null) {
            return face;
        } else {
            face = new Face(letters);
            return face;
        }
    }
    
    public String getCapitalLetters() {
        return capitalLetters;
    }
    
    public String getDisplayLetters() {
        return displayLetters;
    }
    
    public String toString() {
    	return getDisplayLetters();
    }
}
